@@ -20,8 +20,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with GNU Make; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
- *
- *
  */
 
 #include <config.h>
@@ -36,16 +34,15 @@
 
 struct wave_header wave;
 int verbose = 1;
+static uint8_t audio_buff[2*256*1024]; /* Enough for 1fps, 48kHz ... */
+int silence_sr, silence_bs, silence_ch ; 
+EditList el;
 
 int wav_header( unsigned int bits, unsigned int rate, unsigned int channels, int fd );
 void Usage(char *str);
 
-
-/*
-  Raw write does *not* guarantee to write the entire buffer load if it
-  becomes possible to do so.  This does...
- */
-
+/* Raw write does *not* guarantee to write the entire buffer load if it
+   becomes possible to do so.  This does...  */
 static size_t do_write( int fd, void *buf, size_t count )
 {
 	char *cbuf = buf;
@@ -96,8 +93,6 @@ int wav_header( unsigned int bits, unsigned int rate, unsigned int channels, int
 	return 0;
 }
 
-
-
 static void wav_close(int fd)
 {
         unsigned int size;
@@ -112,7 +107,6 @@ static void wav_close(int fd)
 			mjpeg_error("lseek failed - wav-header is corrupt");
 		}
 		goto EXIT;
-
 	}
 
 	/* Rewind file */
@@ -140,9 +134,7 @@ EXIT:
 	close(fd);
 }
 
-
-EditList el;
-
+/* The typical help output */
 void Usage(char *str)
 {
    fprintf(stderr, "Usage: %s [options] inputfiles\n",str);
@@ -152,25 +144,70 @@ void Usage(char *str)
    fprintf(stderr, "-s/-c         Backwards compatibility options for -o/-f\n");
    fprintf(stderr, "-v num        verbose level [0..2]\n");
    fprintf(stderr, "-I            Ignore unsupported bitrates/bits per sample\n");
+   fprintf(stderr, "-r sr,bs,ch   If the file does not contain any sound generate silence\n");
+   fprintf(stderr, "              sr samplerate 32000,44100,48000 bs bytesize 8,16, ch channes 1/2\n");
+   fprintf(stderr, "-R            The same a -R but now a silence with 44100, 16, 2 is created\n");
  
    exit(0);
 }
 
-static uint8_t audio_buff[2*256*1024]; /* Enough for 1fps, 48kHz ... */
+/* Setting the type of silence, samplerate, bitesize, chanels */
+void set_silence (char *optarg)
+{
+unsigned int u1, u2, u3; 
+int i;
+
+i = (sscanf (optarg, "%i,%i,%i", &u1, &u2, &u3));
+
+if (i != 3) 
+  {
+    mjpeg_error("Wrong number of arguments given to the -r option");
+    exit(1);
+  }
+else /*Setting user chosen values */
+  {
+    if ( (u1 == 32000) || (u1 == 44100) || (u1 == 48000) )
+      silence_sr = u1;
+    else {
+      mjpeg_error("Wrong sampelrate use: 32000,44100,48000, exiting");
+      exit(1); }
+   
+    if ( (u2 == 8) || (u2 == 16) )
+      silence_bs = u2;
+    else {
+      mjpeg_error("Wrong audio bitsize use 8 or 16, exiting");
+      exit(1); }
+
+    if ( (u3 == 1) || (u3 == 2) )
+      silence_ch = u3;
+    else {
+      mjpeg_error("Wrong number of chanels use 1 or 2, exiting");
+      exit(1); }
+  }
+
+mjpeg_info("Setting silence to %i sampelrate, %i bitsize, %i chanels",
+     silence_sr, silence_bs, silence_ch);
+
+}
 
 int
 main(argc, argv)
 int     argc;
 char    **argv;
 {
-	int n,f;
+	int n,f,i;
 	int res;
 	int warned = 0;
 	int start_frame = 0;
 	int num_frames = -1;
         int ignore_bitrate = 0;
+	int fragments;
+	char zeros[8];
+   
+silence_sr=0; /* silence_sr is use for detecting if the -r option is used */
+silence_bs=0; silence_ch=0; 
 
-    while( (n=getopt(argc,argv,"v:s:c:o:f:I")) != EOF)
+    while( (n=getopt(argc,argv,"v:s:r:Rc:o:f:I")) != EOF)
     {
         switch(n) {
 
@@ -183,6 +220,15 @@ char    **argv;
 	   case 's':
 		start_frame = atoi(optarg);
 		break;
+	   case 'r':
+                set_silence(optarg);
+                break;
+	   case 'R':
+		{
+		  silence_sr=44100;
+		  silence_bs=16;
+		  silence_ch=2;
+		}
            case 'f':
 	   case 'c':
 		num_frames = atoi(optarg);
@@ -196,8 +242,6 @@ char    **argv;
     }
 
     /* Open editlist */
-
-
 	if( argc-optind < 1)
 		Usage(argv[0]);
 
@@ -207,77 +251,119 @@ char    **argv;
 
     if(!el.has_audio)
     {
-        mjpeg_error("Input file(s) have no audio");
-        exit(1);
-    }
-
-    if(!ignore_bitrate) 
-      {
-        if(el.audio_bits!=16)
-          {
-            mjpeg_error("Input file(s) must have 16 bit audio!");
-            exit(1);
-          }
-        
-        switch (el.audio_rate) {
-        case 48000 : 
-        case 44100 :
-        case 32000 :
-          break;
-        default:
-          mjpeg_error("Audio rate is %ld Hz",el.audio_rate);
-          mjpeg_error("Audio rate must be 32000, 44100 or 48000 !");
+      if (silence_sr == 0) 
+        {   
+          mjpeg_error("Input file(s) have no audio, use the -r option to generate silence"); 
           exit(1);
         }
-      }
+     else
+        {
+
+  mjpeg_info("mjpeg_framerate %f ", el.video_fps);  
 
-    switch (el.audio_chans) {
-    case 1:
-    case 2:
-      break;
-      
-    default:
-      mjpeg_error("Audio channels %d not allowed",el.audio_chans);
-      exit(1);
-    }
- 
 	/* Create wav header (skeleton) on stdout ... */
-	wav_header( el.audio_bits, el.audio_rate, el.audio_chans, 1);
+        /* audio_bits, audio_rate, audio_chans */
+	wav_header( silence_bs, silence_sr, silence_ch, 1);
 	/* skip to the start frame if specified */
-	f = 0;
+        f = 0;
 	if (start_frame != 0)
 		f = start_frame;
 	/* num_frames = -1, read em all, else read specified # */
 	if ( num_frames == -1)
 		num_frames = el.video_frames;
 
+	fragments = silence_sr / el.video_fps;
+       
+	/* Here is a dirty hack for the number of fragments we put out 
+	   I dont know a other way how to put out a certain number of \x00
+           So we put always 4bytes of 00h out and reduce the number of 
+	   fragments. Works perfect for PAL framerate. But in NTSC there is 
+	   a round problem. At 44k1Hz, 16bs 2ch 0,47 samples/frame
+           That means that we have on second offset per hour of film  */
+        if (silence_bs == 8)
+          fragments = fragments / 2;
+        if (silence_ch == 1)
+          fragments = fragments / 2;
+
 	/* Stream out audio wav-style... in per-frame chunks */
 	for( ; f < num_frames; ++f )
-	{
-		n = el_get_audio_data(audio_buff, f, &el, 0);
-		if( n < 0 )
-		{
-			mjpeg_error_exit1( "%s: Couldn't get audio for frame %d!", argv[0], f );
-		}
-		if( n != 0 )
-		{
-			res = do_write( 1, audio_buff, n );
-			if( res != n )
-			{
-				mjpeg_error_exit1( "%s: Couldn't write audio for frame %d!", argv[0], f );
-				exit(1);
-			}
-		}
+	  {
+	    for ( i = 0; i < fragments ; i++) 
+	 	do_write (1, "\x0000000", 4);
+	  }
 
-		else if( f < num_frames && ! warned )
-		{
-			mjpeg_warn( "%s: audio ends early at frame %d", argv[0], f );
-			warned = 1;
-		}
+        wav_close(1);
+        exit(0);
+        }
+    }
+
+    if (!ignore_bitrate) 
+      {
+        if(el.audio_bits!=16)
+          {
+		  mjpeg_error("Input file(s) must have 16 bit audio!");
+		  exit(1);
+	  }
+
+	switch (el.audio_rate) {
+		case 48000 : 
+		case 44100 :
+		case 32000 :
+			break;
+		default:
+			mjpeg_error("Audio rate is %ld Hz",el.audio_rate);
+			mjpeg_error("Audio rate must be 32000, 44100 or 48000 !");
+			exit(1);
 	}
+      }
 
-	wav_close(1);
-	exit(0);
+    switch (el.audio_chans) {
+	    case 1:
+	    case 2:
+		    break;
+
+	    default:
+		    mjpeg_error("Audio channels %d not allowed",el.audio_chans);
+		    exit(1);
+    }
+
+    /* Create wav header (skeleton) on stdout ... */
+    wav_header( el.audio_bits, el.audio_rate, el.audio_chans, 1);
+    /* skip to the start frame if specified */
+    f = 0;
+    if (start_frame != 0)
+	    f = start_frame;
+    /* num_frames = -1, read em all, else read specified # */
+    if ( num_frames == -1)
+	    num_frames = el.video_frames;
+
+    /* Stream out audio wav-style... in per-frame chunks */
+    for( ; f < num_frames; ++f )
+    {
+	    n = el_get_audio_data(audio_buff, f, &el, 0);
+	    if( n < 0 )
+	    {
+		    mjpeg_error_exit1( "%s: Couldn't get audio for frame %d!", argv[0], f );
+	    }
+	    if( n != 0 )
+	    {
+		    res = do_write( 1, audio_buff, n );
+		    if( res != n )
+		    {
+			    mjpeg_error_exit1( "%s: Couldn't write audio for frame %d!", argv[0], f );
+			    exit(1);
+		    }
+	    }
+
+	    else if( f < num_frames && ! warned )
+	    {
+		    mjpeg_warn( "%s: audio ends early at frame %d", argv[0], f );
+		    warned = 1;
+	    }
+    }
+
+    wav_close(1);
+    exit(0);
 }
 
 
