@@ -21,7 +21,6 @@
  *      'A': AVI with fields exchanged
  *      'j': JPEG image(s)
  *      'q': quicktime (if compiled with quicktime support)
- *      'm': movtar (if compiled with movtar support)
  *      Hint: If your AVI video looks strange, try 'A' instead 'a'
  *      and vice versa.
  *		 
@@ -99,6 +98,12 @@
  *      format and need a specific filesize limit, for example to be
  *      able to burn the video files to CD (650 MB).
  *
+ *   --max-file-frames --- The maximum number of frames per video file
+ *      Intended for those that would like to record video in MJPEG
+ *      format and need a specific number of frames per file, for
+ *      example to be able to perform easy file and frame number
+ *      arithmetics when cutting video manually.
+ *
  *   --file-flush  --- How often (in frames) the current output 
  *       file (if any) should be flushed to disk.  (default:60).
  *       Set to 0 if your chosen file-system and system tuning
@@ -222,12 +227,7 @@ static void Usage(char *progname)
 #else
            " "
 #endif
-#ifdef HAVE_LIBMOVTAR
-           "m"
-#else
-           " "
-#endif
-           "]         Format AVI/Quicktime/movtar\n");
+           "]         Format AVI/Quicktime\n");
 	fprintf(stderr, "  -i/--input [pPnNsStTfa]     Input Source\n");
 	fprintf(stderr, "  -d/--decimation num         Decimation (either 1,2,4 or two digit number)\n");
 	fprintf(stderr, "  -g/--geometry WxH+X+Y       X-style geometry string (part of 768/720x576/480)\n");
@@ -253,6 +253,7 @@ static void Usage(char *progname)
 	fprintf(stderr, "  --software-encoding         Use software JPEG-encoding (for BTTV-capture)\n");
 	fprintf(stderr, "  --num-procs num             Number of encoding processes (default: 1)\n");
 	fprintf(stderr, "  --max-file-size num         Maximum size per file (in MB)\n");
+	fprintf(stderr, "  --max-file-frames num       Maximum number of frames per file\n");
 	fprintf(stderr, "  --file-flush num            Flush capture file to disk every num frames\n");
 
 	fprintf(stderr, "  -v/--verbose [012]          verbose level (default: 0)\n");
@@ -579,32 +580,15 @@ static int set_option(const char *name, char *value)
 		if(value[0]!='a' && value[0]!='A' && value[0]!='j'
 #ifdef HAVE_LIBQUICKTIME
                    && value[0]!='q'
-#endif
-#ifdef HAVE_LIBMOVTAR
-                   && value[0]!='m'
 #endif
                    )
 		{
-			mjpeg_error("Format (-f/--format) must be j, a"
-#if !defined(HAVE_LIBMOVTAR) && !defined(HAVE_LIBQUICKTIME)
-                           " or"
+#ifdef	HAVE_LIBQUICKTIME
+		mjpeg_error("Format (-f/--format) must be j, a, A or q");
 #else
-                           ","
+		mjpeg_error("Format (-f/--format) must be j, a or A");
 #endif
-                           " A"
-#ifdef HAVE_LIBQUICKTIME
-#ifdef HAVE_LIBMOVTAR
-                           ","
-#else
-                           " or"
-#endif
-                           " q"
-#endif
-#ifdef HAVE_LIBMOVTAR
-                           " or m"
-#endif
-                           );
-			nerr++;
+		nerr++;
 		}
 	}
 	else if (strcmp(name, "input")==0 || strcmp(name, "i")==0)
@@ -842,6 +826,10 @@ static int set_option(const char *name, char *value)
 	{
 		info->max_file_size_mb = atoi(optarg);
 	}
+	else if (strcmp(name, "max-file-frames")==0)
+	{
+		info->max_file_frames = atoi(optarg);
+	}
 	else if (strcmp(name, "file-flush")==0)
 	{
 		info->flush_count = atoi(optarg);
@@ -888,6 +876,7 @@ static void check_command_line_options(int argc, char *argv[])
 		{"software-encoding",0,0,0},   /* --software-encoding    */
 		{"num-procs"        ,1,0,0},   /* --num-procs            */
 		{"max-file-size"    ,1,0,0},   /* --max-file-size        */
+		{"max-file-frames"  ,1,0,0},   /* --max-file-frames      */
 		{"file-flush"       ,1,0,0},   /* --file-flush           */
 		{"frequency"        ,1,0,0},   /* --frequency/-F         */
 		{0,0,0,0}
@@ -970,10 +959,6 @@ static void check_command_line_options(int argc, char *argv[])
         /* Try to guess the file type by looking at the extension. */
         if(info->video_format == '\0') {
             if((dotptr = strrchr(argv[optind], '.'))) {
-#ifdef HAVE_LIBMOVTAR
-                if(!strcasecmp(dotptr+1, "tar") || !strcasecmp(dotptr+1, "movtar"))
-                    info->video_format = 'm';
-#endif
 #ifdef HAVE_LIBQUICKTIME
                 if(!strcasecmp(dotptr+1, "mov") || !strcasecmp(dotptr+1, "qt")
                     || !strcasecmp(dotptr+1, "moov")) info->video_format = 'q';
@@ -989,9 +974,16 @@ static void check_command_line_options(int argc, char *argv[])
 static void lavrec_print_properties(void)
 {
 	const char *source;
+	char *tmsg;
 	mjpeg_info("Recording parameters:");
-	mjpeg_info("Output format:      %s",info->video_format=='q'?"Quicktime":
-		(info->video_format=='m'?"Movtar":(info->video_format=='j'?"JPEG":"AVI")));
+	if (info->video_format=='q')
+	   tmsg = "Quicktime";
+	else if (info->video_format=='j')
+	   tmsg = "JPEG";
+	else
+	   tmsg = "AVI";
+	mjpeg_info("Output format:      %s", tmsg);
+
 	switch(input_source)
 	{
 		case 'p': source = info->software_encoding?"BT8x8 1st input PAL\n":"Composite PAL\n"; break;
