@@ -26,7 +26,8 @@
  * design.
  *
  */
-/* Modifications and enhancements (C) 2000/2001 Andrew Stevens */
+
+/* Modifications and enhancements (C) 2000,2001,2002,2003 Andrew Stevens */
 
 /* These modifications are free software; you can redistribute it
  *  and/or modify it under the terms of the GNU General Public License
@@ -44,126 +45,24 @@
  * 02111-1307, USA.
  *
  */
-
+
 #include "config.h"
 #include <math.h>
 #include <limits.h>
 #include "mjpeg_types.h"
 #include "mjpeg_logging.h"
-#include "global.h"
+#include "mpeg2syntaxcodes.h"
+#include "tables.h"
 #include "simd.h"
 #include "fastintfns.h"
-
+#include "mpeg2encoder.hh"
+#include "picture.hh"
 #include "ratectl.hh"
+#include "quantize.hh"
 
 /* private prototypes */
 
-static void calc_actj ( Picture &picture, double *act_sum, double *var_sum);
 
-#ifdef JUNK
-/*Constant bit-rate rate control variables */
-/* X's global complexity (Chi! not X!) measures.
-   Actually: average quantisation * bits allocated in *previous* frame
-   N.b. the choice of measure is *not* arbitrary.  The feedback bit rate
-   control gets horribly messed up if it is *not* proportionate to bit demand
-   i.e. bits used scaled for quantisation.
-   d's  virtual reciever buffer fullness
-   r - Rate control feedback gain (in bits/frame)  I.e if the "virtual
-   buffer" is x% full and
-*/
-
-double Xi = 0, Xp = 0, Xb = 0;
-int r;
-int	d0i = 0, d0pb = 0, d0p = 0, d0b = 0;
-
-
-/* R - Remaining bits available in the next one second period.
-   T - Target bits for current frame q
-   d - Current d for quantisation purposes updated using 
-   scaled difference of target bit usage and actual usage
-   NOTE: We use double's for these as 48 bits will suffice and
-   there's some tricky float arithmetic required/
-
-*/
-static int32_t R;
-static int32_t per_pict_bits;
-static int     fields_in_gop;
-static double  field_rate;
-static int     fields_per_pict;
-static int32_t buffer_variation = 0;
-static int d;
-static int32_t T;
-static int64_t bits_transported = 0LL;
-static int64_t bits_used = 0LL;
-static int32_t gop_buffer_correction;
-static int32_t pict_base_bits;
-
-static int32_t I_pict_base_bits;
-static int32_t B_pict_base_bits;
-static int32_t P_pict_base_bits;
-
-/* bitcnt_EOP - Position in generated bit-stream for latest
-				end-of-picture Comparing these values with the
-				bit-stream position for when the picture is due to be
-				displayed allows us to see what the vbv buffer is up
-				to.
-
-*/
-
-static int64_t bitcnt_EOP = 0;
-static int frame_overshoot_margin = 0;
-static int undershoot_carry;
-static double overshoot_gain;
-
-/*
-  actsum - Total activity (sum block variances) in frame
-  actcovered - Activity macroblocks so far quantised (used to
-               fine tune quantisation to avoid starving highly
-			   active blocks appearing late in frame...) UNUSED
-  avg_act - Current average activity...
-*/
-static double actsum;
-static double actcovered;
-static double sum_avg_act = 0.0;
-static double avg_act;
-static double avg_var;
-static double sum_avg_var = 0.0;
-static double sum_avg_quant = 0.0;
-
-static int Ni, Np, Nb;
-static int64_t S;
-
-/* Note: eventually we may wish to tweak these to suit image content */
-static double Ki;    	/* Down-scaling of I/B/P-frame complexity */
-static double Kb;	    /* relative to others in bit-allocation   */
-static double Kp;	    /* calculations.  We only need 2 but have all
-							   3 for readability */
-
-
-static int min_d,max_d;
-static int min_q, max_q;
-
-static double avg_KI = 5.0;	      /* TODO: These values guesstimates        */
-static double avg_KB = 10.0*2.0;  /* for MPEG-1, may need tuning for MPEG-2	*/
-static double avg_KP = 10.0*2.0;
-#define K_AVG_WINDOW_I 4.0
-#define K_AVG_WINDOW_P   10.0
-#define K_AVG_WINDOW_B   20.0
-static double bits_per_mb;
-
-static double sum_vbuf_Q = 0.0;
-static bool fast_tune = false;
-static bool first_gop = true;
-static bool first_B = false;
-static bool first_P = false;
-static bool first_I = false;
-
-/* Scaling and clipping of quantisation factors
-   One floating point used in predictive calculations.  
-   The second integerises the results of the first for use in the
-   actual stream...
-*/
-#endif
 
 static double scale_quantf( int q_scale_type, double quant )
 {
@@ -211,7 +110,7 @@ static double scale_quantf( int q_scale_type, double quant )
 }
 
 
-int scale_quant( int q_scale_type , double quant )
+int RateCtl::ScaleQuant( int q_scale_type , double quant )
 {
 	int iquant;
 	
@@ -241,7 +140,7 @@ int scale_quant( int q_scale_type , double quant )
 	return iquant;
 }
 
-double inv_scale_quant( int q_scale_type, int raw_code )
+double RateCtl::InvScaleQuant( int q_scale_type, int raw_code )
 {
 	int i;
 	if( q_scale_type )
@@ -255,6 +154,11 @@ double inv_scale_quant( int q_scale_type, int raw_code )
 		return ((double)raw_code);
 }
 
+RateCtl::RateCtl( EncoderParams &_encparams ) :
+	encparams( _encparams )
+{
+}
+
 /*****************************
  *
  * On-the-fly rate controller.  The constructor sets up the initial
@@ -263,7 +167,8 @@ double inv_scale_quant( int q_scale_type, int raw_code )
  * tuned anyway so these values are not too critical.
  *
  ****************************/
-OnTheFlyRateCtl::OnTheFlyRateCtl()
+OnTheFlyRateCtl::OnTheFlyRateCtl(EncoderParams &encparams ) :
+	RateCtl(encparams)
 {
 	buffer_variation = 0;
 	bits_transported = 0;
@@ -280,12 +185,8 @@ OnTheFlyRateCtl::OnTheFlyRateCtl()
 	   real encoding... alternative a config file should  be written!
 	*/
 
-	avg_KI = 5.0; 
-	avg_KB = 10.0*2.0;
-	avg_KP = 10.0*2.0;
 	sum_avg_quant = 0.0;
 
-
 }
 
 /*********************
@@ -302,22 +203,21 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 	   guesstimates of for initial quantisation pessimistic...
 	*/
 	bits_transported = bits_used = 0;
-	field_rate = 2*ctl_decode_frame_rate;
-	fields_per_pict = opt_fieldpic ? 1 : 2;
-	if( opt_still_size > 0 )
+	field_rate = 2*encparams.decode_frame_rate;
+	fields_per_pict = encparams.fieldpic ? 1 : 2;
+	if( encparams.still_size > 0 )
 	{
-		avg_KI *= 1.5;
-		per_pict_bits = opt_still_size * 8;
-		R = opt_still_size * 8;
+		per_pict_bits = encparams.still_size * 8;
+		R = encparams.still_size * 8;
 	}
 	else
 	{
 		per_pict_bits = 
-			static_cast<int32_t>(opt_fieldpic
-								 ? opt_bit_rate / field_rate
-								 : opt_bit_rate / ctl_decode_frame_rate
+			static_cast<int32_t>(encparams.fieldpic
+								 ? encparams.bit_rate / field_rate
+								 : encparams.bit_rate / encparams.decode_frame_rate
 				);
-		R = static_cast<int32_t>(opt_bit_rate);
+		R = static_cast<int32_t>(encparams.bit_rate);
 	}
 
 	/* Everything else already set or adaptive */
@@ -325,6 +225,23 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 		return;
 
 	first_gop = true;
+	K_AVG_WINDOW_I = 2.0;
+    switch( encparams.M )
+    {
+    case 1 : // P
+        K_AVG_WINDOW_P = 8.0;
+        K_AVG_WINDOW_B = 1.0; // dummy
+        break;
+    case 2 : // BP
+        K_AVG_WINDOW_P = 4.0;
+        K_AVG_WINDOW_B = 4.0;
+        break;
+    default: // BBP
+        K_AVG_WINDOW_P = 3.0;
+        K_AVG_WINDOW_B = 7.0;
+        break;
+    }        
+
 
 	/* Calculate reasonable margins for variation in the decoder
 	   buffer.  We assume that having less than 5 frame intervals
@@ -337,7 +254,7 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 	   Currently, for a 1-frame sized margin gain is set to recover
 	   an undershoot in half a second
 	*/
-	if( opt_still_size > 0 )
+	if( encparams.still_size > 0 )
 	{
 		undershoot_carry = 0;
 		overshoot_gain = 1.0;
@@ -345,12 +262,12 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 	else
 	{
 		int buffer_safe = 3 * per_pict_bits ;
-		undershoot_carry = (ctl_video_buffer_size - buffer_safe)/6;
+		undershoot_carry = (encparams.video_buffer_size - buffer_safe)/6;
 		if( undershoot_carry < 0 )
 			mjpeg_error_exit1("Rate control can't cope with a video buffer smaller 4 frame intervals");
-		overshoot_gain =  opt_bit_rate / (ctl_video_buffer_size-buffer_safe);
+		overshoot_gain =  encparams.bit_rate / (encparams.video_buffer_size-buffer_safe);
 	}
-	bits_per_mb = (double)opt_bit_rate / (mb_per_pict);
+	bits_per_mb = (double)encparams.bit_rate / (encparams.mb_per_pict);
 
 	/*
 	  Reaction paramer - i.e. quantisation feedback gain relative
@@ -362,10 +279,10 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 	  For stills we set it a higher so corrections take place
 	  more rapidly *within* a single frame.
 	*/
-	if( opt_still_size > 0 )
-		r = (int)floor(2.0*opt_bit_rate/ctl_decode_frame_rate);
+	if( encparams.still_size > 0 )
+		r = (int)floor(2.0*encparams.bit_rate/encparams.decode_frame_rate);
 	else
-		r = (int)floor(4.0*opt_bit_rate/ctl_decode_frame_rate);
+		r = (int)floor(4.0*encparams.bit_rate/encparams.decode_frame_rate);
 
 
 	/* Set the virtual buffers for per-frame rate control feedback to
@@ -373,14 +290,14 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 	   or a "reasonable" quantisation (6.0) if not.
 	*/
 
-	init_quant = (ctl_quant_floor > 0.0 ? ctl_quant_floor : 6.0);
-	d0p = d0b = d0pb = d0i = static_cast<int>(init_quant * r / 62.0);
+	init_quant = (encparams.quant_floor > 0.0 ? encparams.quant_floor : 6.0);
+	d0p = d0b = d0i = static_cast<int>(init_quant * r / 62.0);
 
 	next_ip_delay = 0.0;
 	decoding_time = 0.0;
 
 
-#ifdef OUTPUT_STAT
+#ifdef OUTPUTr_STAT
 	fprintf(statfile,"\nrate control: sequence initialization\n");
 	fprintf(statfile,
 			" initial global complexity measures (I,P,B): Xi=%.0f, Xp=%.0f, Xb=%.0f\n",
@@ -397,9 +314,9 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 
 void OnTheFlyRateCtl::InitGOP( int np, int nb)
 {
-	Np = opt_fieldpic ? 2*np+1 : 2*np;
-	Nb = opt_fieldpic ? 2*nb : 2*nb;
-	Ni = opt_fieldpic ? 1 : 2;
+	Np = encparams.fieldpic ? 2*np+1 : 2*np;
+	Nb = encparams.fieldpic ? 2*nb : 2*nb;
+	Ni = encparams.fieldpic ? 1 : 2;
 	fields_in_gop = Ni + Nb + Np;
 
 	/*
@@ -415,7 +332,7 @@ void OnTheFlyRateCtl::InitGOP( int np, int nb)
 	   for each one.  They're all I-frames so each stills is a GOP too.
 	*/
 
-	if( first_gop || opt_still_size > 0)
+	if( first_gop || encparams.still_size > 0)
 	{
 		mjpeg_debug( "FIRST GOP INIT");
 		fast_tune = true;
@@ -427,14 +344,14 @@ void OnTheFlyRateCtl::InitGOP( int np, int nb)
 	}
 	else
 	{
+		mjpeg_debug( "REST GOP INIT" );
 		double recovery_fraction = field_rate/(overshoot_gain * fields_in_gop);
 		double recovery_gain = 
 			recovery_fraction > 1.0 ? 1.0 : overshoot_gain * recovery_fraction;
 		int available_bits = 
-			static_cast<int>( (opt_bit_rate+buffer_variation*recovery_gain)
+			static_cast<int>( (encparams.bit_rate+buffer_variation*recovery_gain)
 							  * fields_in_gop/field_rate);
 		double Xsum = Ni*Xi+Np*Xp+Nb*Xb;
-		mjpeg_debug( "REST GOP INIT" );
 		I_pict_base_bits = (int32_t)(fields_per_pict*available_bits*Xi/Xsum);
 		P_pict_base_bits = (int32_t)(fields_per_pict*available_bits*Xp/Xsum);
 		B_pict_base_bits = (int32_t)(fields_per_pict*available_bits*Xb/Xsum);
@@ -452,15 +369,12 @@ void OnTheFlyRateCtl::InitGOP( int np, int nb)
 
 
 /* Step 1: compute target bits for current picture being coded */
-void OnTheFlyRateCtl::InitPict(Picture &picture)
+void OnTheFlyRateCtl::InitPict(Picture &picture, int64_t bitcount_SOP)
 {
-	double avg_K = 0.0;
 	double target_Q;
 	double current_Q;
-	double Si, Sp, Sb;
 	int available_bits;
 	double Xsum,varsum;
-	bool no_avg_K = false;
 
 	/* TODO: A.Stevens  Nov 2000 - This modification needs testing visually.
 
@@ -476,9 +390,9 @@ void OnTheFlyRateCtl::InitPict(Picture &picture)
 	   We use this to try to predict the activity of each frame.
 	*/
 	
-	calc_actj(picture, &actsum, &varsum );
-	avg_act = actsum/(double)(mb_per_pict);
-	avg_var = varsum/(double)(mb_per_pict);
+	picture.ActivityMeasures( actsum, varsum );
+	avg_act = actsum/(double)(encparams.mb_per_pict);
+	avg_var = varsum/(double)(encparams.mb_per_pict);
 	sum_avg_act += avg_act;
 	sum_avg_var += avg_var;
 	actcovered = 0.0;
@@ -511,7 +425,7 @@ void OnTheFlyRateCtl::InitPict(Picture &picture)
 	*/
 
 	
-	if( opt_still_size > 0 )
+	if( encparams.still_size > 0 )
 		available_bits = per_pict_bits;
 	else
 	{
@@ -522,7 +436,7 @@ void OnTheFlyRateCtl::InitPict(Picture &picture)
 							    * overshoot_gain
 				);
 		available_bits = 
-			static_cast<int>( (opt_bit_rate+feedback_correction)
+			static_cast<int>( (encparams.bit_rate+feedback_correction)
 							  * fields_in_gop/field_rate
 							  );
 	}
@@ -540,49 +454,37 @@ void OnTheFlyRateCtl::InitPict(Picture &picture)
 		*/
 
 		d = d0i;
-		avg_K = avg_KI;
-		Si = avg_K*actsum;
-		no_avg_K = first_I;
 		if( first_I )
 		{
 			T = (int32_t)(fields_per_pict*available_bits/(Ni+(Np/1.7)+Nb/(2.0*1.7)));
 		}
 		else
 		{
-			T = (int32_t)(fields_per_pict*available_bits*Si/Xsum);
+			T = (int32_t)(fields_per_pict*available_bits*Xi/Xsum);
 		}
 		pict_base_bits = I_pict_base_bits;
 		break;
 	case P_TYPE:
 		d = d0p;
-		avg_K = avg_KP;
-		Sp = (2*Xp + avg_K * actsum)/3.0;	 /* Damp P-frame response a little
-											as really big jumps will probably
-											make avg_K inaccurate */
-		no_avg_K = first_P;
 		if( first_P )
 		{
 			T = (int32_t)(fields_per_pict*available_bits/(Np+Nb/2.0));
 		}
 		else
 		{
-			T = (int32_t)(fields_per_pict*available_bits*Sp/Xsum);
+			T = (int32_t)(fields_per_pict*available_bits*Xp/Xsum);
 		}
 		pict_base_bits = P_pict_base_bits;
 		break;
 	case B_TYPE:
 		d = d0b;
-		avg_K = avg_KB;
-		Sb = Xb;				/* We don't want B-frames to be too
-								   responsive...*/
-		no_avg_K = first_B;
 		if( first_B )
 		{
 			T =  (int32_t)(fields_per_pict*available_bits/(Nb+Np*2.0));
 		}
 		else
 		{
-			T =  (int32_t)(fields_per_pict*available_bits*Sb/Xsum);
+			T =  (int32_t)(fields_per_pict*available_bits*Xb/Xsum);
 		}
 		pict_base_bits = B_pict_base_bits;
 
@@ -596,10 +498,12 @@ void OnTheFlyRateCtl::InitPict(Picture &picture)
 	   limit any individual frame to 3/4's of the buffer.
 	*/
 
-	T = intmin( T, ctl_video_buffer_size*3/4 );
+	T = intmin( T, encparams.video_buffer_size*3/4 );
 
-	mjpeg_debug( "I=%d P=%d B=%d", I_pict_base_bits, P_pict_base_bits, B_pict_base_bits );
-	mjpeg_debug( "T=%05d A=%06d D=%06d (%06d) ", (int)T/8, (int)available_bits/8, (int)buffer_variation/8, (int)(buffer_variation + gop_buffer_correction)/8 );
+	mjpeg_debug( "Frame %c T=%05d A=%06d  Xi=%.2f Xp=%.2f Xb=%.2f", 
+                 pict_type_char[picture.pict_type],
+                 (int)T/8, (int)available_bits/8, 
+                 Xi, Xp,Xb );
 
 
 	/* 
@@ -615,7 +519,7 @@ void OnTheFlyRateCtl::InitPict(Picture &picture)
 	   bit-allocation (which *won't* add up very well).
 	*/
 
-	mjpeg_debug( "PBB=%d PPB=%d", pict_base_bits, per_pict_bits );
+	//mjpeg_debug( "PBB=%d PPB=%d", pict_base_bits, per_pict_bits );
 	gop_buffer_correction += (pict_base_bits-per_pict_bits);
 
 
@@ -629,7 +533,7 @@ void OnTheFlyRateCtl::InitPict(Picture &picture)
 	*/
 	T = intmax( T, 4000 );
 
-	if( opt_still_size > 0 && opt_vbv_buffer_still_size )
+	if( encparams.still_size > 0 && encparams.vbv_buffer_still_size )
 	{
 		/* If stills size must match then target low to ensure no
 		   overshoot.
@@ -640,37 +544,12 @@ void OnTheFlyRateCtl::InitPict(Picture &picture)
 	}
 
 
-	current_Q = scale_quant(picture.q_scale_type,62.0*d / r);
-	if( no_avg_K )
-		target_Q = current_Q;
-	else
-		target_Q = scale_quantf(picture.q_scale_type, 
-								avg_K * avg_act *(mb_per_pict) / T);
+	current_Q = ScaleQuant(picture.q_scale_type,62.0*d / r);
 
 	picture.avg_act = avg_act;
 	picture.sum_avg_act = sum_avg_act;
 
-	/*  If guesstimated target quantisation requirement is much different
-		than that suggested by the virtual buffer we reset the virtual
-		buffer to set the guesstimate as initial quantisation.
-
-		This effectively speeds up rate control response when sudden
-		changes in content occur
-	*/
-
-
-	if ( 62.0*d / r  < target_Q / 2.0  )
-	{
-		d = (int) (target_Q  * r / (62.0));
-	}
-
-	if ( 62.0*d / r  > target_Q * 2.0  )
-	{
-		d = (int) (d+(target_Q  * r / 62.0))/2;
-	}
-
-	S = bitcount();
-
+	S = bitcount_SOP;
 
 #ifdef OUTPUT_STAT
 	fprintf(statfile,"\nrate control: start of picture\n");
@@ -680,74 +559,14 @@ void OnTheFlyRateCtl::InitPict(Picture &picture)
 
 
 
-static void calc_actj(Picture &picture, double *act_sum, double *var_sum)
-{
-	int i,j,k,l;
-	double actj,sum;
-	double varsum;
-	uint16_t *i_q_mat;
-	int blksum;
-	sum = 0.0;
-	varsum = 0.0;
-	k = 0;
-
-	for (j=0; j<opt_enc_height2; j+=16)
-		for (i=0; i<opt_enc_width; i+=16)
-		{
-			/* A.Stevens Jul 2000 Luminance variance *has* to be a
-			   rotten measure of how active a block in terms of bits
-			   needed to code a lossless DCT.  E.g. a half-white
-			   half-black block has a maximal variance but pretty
-			   small DCT coefficients.
-
-			   So.... instead of luminance variance as used in the
-			   original we use the absolute sum of DCT coefficients as
-			   our block activity measure.  */
-
-			varsum += (double)picture.mbinfo[k].final_me.var;
-			if( picture.mbinfo[k].final_me.mb_type  & MB_INTRA )
-			{
-				i_q_mat = i_intra_q;
-				/* Compensate for the wholly disproprotionate weight
-				 of the DC coefficients.  Shold produce more sensible
-				 results...  yes... it *is* an mostly empirically derived
-				 fudge factor ;-)
-				*/
-				blksum =  -80*COEFFSUM_SCALE;
-			}
-			else
-			{
-				i_q_mat = i_inter_q;
-				blksum = 0;
-			}
-			/* It takes some bits to code even an entirely zero block...
-			   It also makes a lot of calculations a lot better conditioned
-			   if it can be guaranteed that activity is always distinctly
-			   non-zero.
-			 */
-
-
-			for( l = 0; l < 6; ++l )
-				blksum += 
-					(*pquant_weight_coeff_sum)
-					( picture.mbinfo[k].RawDCTblocks()[l], i_q_mat ) ;
-			actj = (double)blksum / (double)COEFFSUM_SCALE;
-			if( actj < 12.0 )
-				actj = 12.0;
-
-			picture.mbinfo[k].act = (double)actj;
-			sum += (double)actj;
-			++k;
-		}
-	*act_sum = sum;
-	*var_sum = varsum;
-}
-
 /*
  * Update rate-controls statistics after pictures has ended..
  *
+ * RETURN: The amount of padding necessary for picture to meet syntax or
+ * rate constraints...
  */
-void OnTheFlyRateCtl::UpdatePict(Picture &picture)
+
+int OnTheFlyRateCtl::UpdatePict(Picture &picture, int64_t _bitcount_EOP)
 {
 	double X;
 	double K;
@@ -756,13 +575,14 @@ void OnTheFlyRateCtl::UpdatePict(Picture &picture)
 	int    i;
 	int    Qsum;
 	int frame_overshoot;
-
-	AP = bitcount() - S;
+    int64_t bitcount_EOP = _bitcount_EOP; 
+	AP = bitcount_EOP - S;
 	frame_overshoot = (int)AP-(int)T;
 
 	/* For the virtual buffers for quantisation feedback it is the
 	   actual under/overshoot *including* padding.  Otherwise the
 	   buffers go zero.
+       BUGBUGBUG  should'nt this go after the padding calculation?
 	*/
 	d += frame_overshoot;
 
@@ -772,14 +592,13 @@ void OnTheFlyRateCtl::UpdatePict(Picture &picture)
 	   specified in advance in vbv_buffer_size.
 	*/
 	picture.pad = 0;
-
-	if( opt_still_size > 0 && opt_vbv_buffer_still_size)
+    int padding_bits = 0;
+	if( encparams.still_size > 0 && encparams.vbv_buffer_still_size)
 	{
-		int padding_bytes;
 		if( frame_overshoot > frame_overshoot_margin )
 		{
 			mjpeg_warn( "Rate overshoot: VCD hi-res still %d bytes too large! ", 
-						((int)AP)/8-opt_still_size);
+						((int)AP)/8-encparams.still_size);
 		}
 		
 		//
@@ -790,22 +609,31 @@ void OnTheFlyRateCtl::UpdatePict(Picture &picture)
 		frame_overshoot = frame_overshoot - frame_overshoot_margin;
 		if( frame_overshoot < -2048*8 )
 			frame_overshoot += 1024*8;
-
-		padding_bytes = -frame_overshoot/8;
+        
+        // Make sure we pad nicely to byte alignment
+        if( frame_overshoot < 0 )
+        {
+            padding_bits = (((bitcount_EOP-frame_overshoot)>>3)<<3)-bitcount_EOP;
+            picture.pad = 1;
+        }
+#ifdef JUNK_DONE_ELSEWHERE_NOW
 		if( padding_bytes > 0 )
 		{
 			mjpeg_debug( "Padding still to size: %d extra bytes", padding_bytes );
 			picture.pad = 1;
-			alignbits();
 			for( i = 0; i < padding_bytes/2; ++i )
 			{
-				putbits(0, 16);
+				writer.PutBits(0, 16);
 			}
 		}
-		AP += padding_bytes*8 ;			/* total # of bits in picture */
-		frame_overshoot += padding_bytes*8;
+#endif
 	}
 
+    /* Adjust the various bit counting  parameters for the padding bytes that
+     * will be added */
+    AP += padding_bits ;
+    frame_overshoot += padding_bits;
+    bitcount_EOP += padding_bits;
 
 	/*
 	  Compute the estimate of the current decoder buffer state.  We
@@ -821,15 +649,16 @@ void OnTheFlyRateCtl::UpdatePict(Picture &picture)
 	*/
 
 	
-	bits_used += (bitcount()-prev_bitcount);
-	prev_bitcount = bitcount();
+	bits_used += (bitcount_EOP-prev_bitcount);
+	prev_bitcount = bitcount_EOP;
+    
 	bits_transported += per_pict_bits;
-	mjpeg_debug( "TR=%" PRId64 " USD=%" PRId64 "", bits_transported/8, bits_used/8);
+	//mjpeg_debug( "TR=%" PRId64 " USD=%" PRId64 "", bits_transported/8, bits_used/8);
 	buffer_variation  = (int32_t)(bits_transported - bits_used);
 
 	if( buffer_variation > 0 )
 	{
-		if( ctl_quant_floor > 0 )
+		if( encparams.quant_floor > 0 )
 		{
 			bits_transported = bits_used;
 			buffer_variation = 0;	
@@ -843,7 +672,7 @@ void OnTheFlyRateCtl::UpdatePict(Picture &picture)
 
 
 	Qsum = 0;
-	for( i = 0; i < mb_per_pict; ++i )
+	for( i = 0; i < encparams.mb_per_pict; ++i )
 	{
 		Qsum += picture.mbinfo[i].mquant;
 	}
@@ -853,7 +682,7 @@ void OnTheFlyRateCtl::UpdatePict(Picture &picture)
 	   Its only used for stats display as the integerisation
 	   of the quantisation value makes it rather coarse for use in
 	   estimating bit-demand */
-	AQ = (double)Qsum/(double)mb_per_pict;
+	AQ = (double)Qsum/(double)encparams.mb_per_pict;
 	sum_avg_quant += AQ;
 	
 	/* X (Chi - Complexity!) is an estimate of "bit-demand" for the
@@ -867,15 +696,22 @@ void OnTheFlyRateCtl::UpdatePict(Picture &picture)
 	prediction of quantisation needed to hit a bit-allocation.
 	*/
 
-	X = AP  * AQ; //sum_vbuf_Q /  mb_per_pict;
+	X = AP  * AQ;
+
+    /* To handle longer sequences with little picture content
+       where I, B and P frames are of unusually similar size we
+       insist I frames assumed to be at least one and a half times
+       as complex as typical P frames
+    */
+    if( picture.pict_type == I_TYPE )
+        X = fmax(X, 1.5*Xp);
+
 	K = X / actsum;
 	picture.AQ = AQ;
 	picture.SQ = sum_avg_quant;
 
-	mjpeg_debug( "D=%d R=%d GC=%d", 
-				 buffer_variation/8,
-				 (int)R/8,
-				 gop_buffer_correction/8  );
+	//mjpeg_debug( "D=%d R=%d GC=%d", buffer_variation/8, (int)R/8, 
+    //gop_buffer_correction/8  );
 
 	/* Xi are used as a guesstimate of *typical* frame activities
 	   based on the past.  Thus we don't want anomalous outliers due
@@ -885,44 +721,33 @@ void OnTheFlyRateCtl::UpdatePict(Picture &picture)
 	   similar real-time decay periods based on an assumption of
 	   20-30Hz frame rates.
 	*/
-
 	switch (picture.pict_type)
 	{
 	case I_TYPE:
 		d0i = d;
+        sum_I_size += AP/8.0;
+        ++I_count;
 		if( first_I )
 		{
 			Xi = X;
-			avg_KI = K;
 			first_I = 0;
 		}
 		else
 		{
-			avg_KI = (K + avg_KI * K_AVG_WINDOW_I) / (K_AVG_WINDOW_I+1.0) ;
 			Xi = (X + K_AVG_WINDOW_I*Xi)/(K_AVG_WINDOW_I+1.0);
-
-			/* To handle longer sequences with little picture content
-		   where I, B and P frames are of unusually similar size we
-		   insist I frames assumed to be at least of the same
-		   complexity as two B and a P frame.  This ensures that
-		   *after* a low picture content sequence there won't be a
-		   grossly under-estimated I-frame size that hasn't been
-		   allowed for in the buffer management.
-			*/
-			Xi = Xi < Xp+2.0*Xb ? Xp+2.0*Xb  : Xi;
 		}
 		break;
 	case P_TYPE:
+        sum_P_size += AP/8.0;
+        ++P_count;
 		d0p = d;
 		if( first_P )
 		{
 			Xp = X;
-			avg_KP = K;
 			first_P = 0;
 		}
 		else
 		{
-			avg_KP = (K + avg_KP * K_AVG_WINDOW_P) / (K_AVG_WINDOW_P+1.0) ;
 			if( fast_tune )
 				Xp = (X+Xp*2.0)/3.0;
 			else
@@ -930,16 +755,16 @@ void OnTheFlyRateCtl::UpdatePict(Picture &picture)
 		}
 		break;
 	case B_TYPE:
+        sum_B_size += AP/8.0;
+        ++B_count;
 		d0b = d;
 		if( first_B )
 		{
 			Xb = X;
-			avg_KB = K;
 			first_B = 0;
 		}
 		else 
 		{
-			avg_KB = (K + avg_KB * K_AVG_WINDOW_B) / (K_AVG_WINDOW_B+1.0) ;
 			if( fast_tune )
 			{
 				Xb = (X + Xb * 3.0) / 4.0;
@@ -950,7 +775,16 @@ void OnTheFlyRateCtl::UpdatePict(Picture &picture)
 		break;
 	}
 
-	VbvEndOfPict(picture);
+    mjpeg_debug( "Frame %c A=%6.0f %.2f: I = %6.0f P = %5.0f B = %5.0f",
+                pict_type_char[picture.pict_type],
+                AP/8.0,
+                X,
+                sum_I_size/I_count,
+                sum_P_size/P_count,
+                sum_B_size/B_count );
+
+                
+	VbvEndOfPict(picture, bitcount_EOP);
 
 #ifdef OUTPUT_STAT
 	fprintf(statfile,"\nrate control: end of picture\n");
@@ -968,27 +802,25 @@ void OnTheFlyRateCtl::UpdatePict(Picture &picture)
 	fprintf(statfile," remaining number of B pictures in GOP: Nb=%d\n",Nb);
 	fprintf(statfile," average activity: avg_act=%.1f \n", avg_act );
 #endif
-
+    return padding_bits/8;
 }
 
 /* compute initial quantization stepsize (at the beginning of picture) 
-   ctl_quant_floor != 0 is the VBR case where we set a bitrate as a (high)
+   encparams.quant_floor != 0 is the VBR case where we set a bitrate as a (high)
    maximum and then put a floor on quantisation to achieve a reasonable
    overall size.
  */
-static int init_quant;
 
 int OnTheFlyRateCtl::InitialMacroBlockQuant(Picture &picture)
 {
 	
-	int mquant = scale_quant( picture.q_scale_type, d*62.0/r );
-	init_quant = mquant = intmax(mquant, static_cast<int>(ctl_quant_floor));
-
+	int mquant = ScaleQuant( picture.q_scale_type, d*62.0/r );
+	
 /*
   fprintf(statfile,"rc_start_mb:\n");
   fprintf(statfile,"mquant=%d\n",mquant);
 */
-	return mquant;
+	return intmax(mquant, static_cast<int>(encparams.quant_floor));
 }
 
 
@@ -999,7 +831,7 @@ int OnTheFlyRateCtl::InitialMacroBlockQuant(Picture &picture)
  *
  ************/
 
-int OnTheFlyRateCtl::MacroBlockQuant( const MacroBlock &mb )
+int OnTheFlyRateCtl::MacroBlockQuant( const MacroBlock &mb, int64_t bitcount )
 {
 	int mquant;
 	int lum_variance = mb.BaseLumVariance();
@@ -1019,11 +851,11 @@ int OnTheFlyRateCtl::MacroBlockQuant( const MacroBlock &mb )
 	*/
 
 	double dj = ((double)d) + 
-		((double)(bitcount()-S) - actcovered * ((double)T) / actsum);
+		((double)(bitcount-S) - actcovered * ((double)T) / actsum);
 
 
 	/* scale against dynamic range of mquant and the bits/picture
-	   count.  ctl_quant_floor != 0.0 is the VBR case where we set a
+	   count.  encparams.quant_floor != 0.0 is the VBR case where we set a
 	   bitrate as a (high) maximum and then put a floor on
 	   quantisation to achieve a reasonable overall size.  Not that
 	   this *is* baseline quantisation.  Not adjust for local
@@ -1032,7 +864,7 @@ int OnTheFlyRateCtl::MacroBlockQuant( const MacroBlock &mb )
 	*/
 
 	double Qj = dj*62.0/r;
-	Qj = (Qj > ctl_quant_floor) ? Qj : ctl_quant_floor;
+	Qj = (Qj > encparams.quant_floor) ? Qj : encparams.quant_floor;
 
 	/*  Heuristic: We decrease quantisation for macroblocks
 		with markedly low luminace variance.  This helps make
@@ -1044,20 +876,20 @@ int OnTheFlyRateCtl::MacroBlockQuant( const MacroBlock &mb )
 #ifdef OLD_QUANTISATION_STEARING
 	double N_act =  ( act < avg_act || picture.pict_type == B_TYPE ) ? 
 		1.0 : 
-		(ctl_act_boost*act +  avg_act)/(act + ctl_act_boost*avg_act);
+		(encparams.act_boost*act +  avg_act)/(act + encparams.act_boost*avg_act);
 	act_boost = 1.0/N_act;
 #else
-	if( lum_variance < ctl_boost_var_ceil )
+	if( lum_variance < encparams.boost_var_ceil )
 	{
-		if( lum_variance < ctl_boost_var_ceil/2)
-			act_boost = ctl_act_boost;
+		if( lum_variance < encparams.boost_var_ceil/2)
+			act_boost = encparams.act_boost;
 		else
 		{
-			double max_boost_var = ctl_boost_var_ceil/2;
+			double max_boost_var = encparams.boost_var_ceil/2;
 			double above_max_boost = 
 				(static_cast<double>(lum_variance)-max_boost_var)
 				/ max_boost_var;
-			act_boost = 1.0 + (ctl_act_boost-1.0) * (1.0-above_max_boost);
+			act_boost = 1.0 + (encparams.act_boost-1.0) * (1.0-above_max_boost);
 		}
 	}
 	else
@@ -1065,7 +897,7 @@ int OnTheFlyRateCtl::MacroBlockQuant( const MacroBlock &mb )
 
 #endif	
 	sum_vbuf_Q += scale_quantf(picture.q_scale_type,Qj/act_boost);
-	mquant = scale_quant(picture.q_scale_type,Qj/act_boost) ;
+	mquant = ScaleQuant(picture.q_scale_type,Qj/act_boost) ;
 	
 	/* Update activity covered */
 
@@ -1097,9 +929,9 @@ int OnTheFlyRateCtl::MacroBlockQuant( const MacroBlock &mb )
  * bit-stream.
  */
 
-void OnTheFlyRateCtl::VbvEndOfPict(Picture &picture)
+void OnTheFlyRateCtl::VbvEndOfPict(Picture &picture, int64_t bitcount)
 {
-	bitcnt_EOP = (bitcount()) - BITCOUNT_OFFSET;
+	bitcnt_EOP = bitcount - BITCOUNT_OFFSET;
 
 }
 
@@ -1120,47 +952,47 @@ void OnTheFlyRateCtl::CalcVbvDelay(Picture &picture)
 	/* number of 1/90000 s ticks until next picture is to be decoded */
 	if (picture.pict_type == B_TYPE)
 	{
-		if (opt_prog_seq)
+		if (encparams.prog_seq)
 		{
 			if (!picture.repeatfirst)
-				picture_delay = 90000.0/opt_frame_rate; /* 1 frame */
+				picture_delay = 90000.0/encparams.frame_rate; /* 1 frame */
 			else
 			{
 				if (!picture.topfirst)
-					picture_delay = 90000.0*2.0/opt_frame_rate; /* 2 frames */
+					picture_delay = 90000.0*2.0/encparams.frame_rate; /* 2 frames */
 				else
-					picture_delay = 90000.0*3.0/opt_frame_rate; /* 3 frames */
+					picture_delay = 90000.0*3.0/encparams.frame_rate; /* 3 frames */
 			}
 		}
 		else
 		{
 			/* interlaced */
-			if (opt_fieldpic)
-				picture_delay = 90000.0/(2.0*opt_frame_rate); /* 1 field */
+			if (encparams.fieldpic)
+				picture_delay = 90000.0/(2.0*encparams.frame_rate); /* 1 field */
 			else
 			{
 				if (!picture.repeatfirst)
-					picture_delay = 90000.0*2.0/(2.0*opt_frame_rate); /* 2 flds */
+					picture_delay = 90000.0*2.0/(2.0*encparams.frame_rate); /* 2 flds */
 				else
-					picture_delay = 90000.0*3.0/(2.0*opt_frame_rate); /* 3 flds */
+					picture_delay = 90000.0*3.0/(2.0*encparams.frame_rate); /* 3 flds */
 			}
 		}
 	}
 	else
 	{
 		/* I or P picture */
-		if (opt_fieldpic)
+		if (encparams.fieldpic)
 		{
 			if(picture.topfirst && (picture.pict_struct==TOP_FIELD))
 			{
 				/* first field */
-				picture_delay = 90000.0/(2.0*opt_frame_rate);
+				picture_delay = 90000.0/(2.0*encparams.frame_rate);
 			}
 			else
 			{
 				/* second field */
 				/* take frame reordering delay into account */
-				picture_delay = next_ip_delay - 90000.0/(2.0*opt_frame_rate);
+				picture_delay = next_ip_delay - 90000.0/(2.0*encparams.frame_rate);
 			}
 		}
 		else
@@ -1170,32 +1002,32 @@ void OnTheFlyRateCtl::CalcVbvDelay(Picture &picture)
 			picture_delay = next_ip_delay;
 		}
 
-		if (!opt_fieldpic || 
+		if (!encparams.fieldpic || 
 			picture.topfirst!=(picture.pict_struct==TOP_FIELD))
 		{
 			/* frame picture or second field */
-			if (opt_prog_seq)
+			if (encparams.prog_seq)
 			{
 				if (!picture.repeatfirst)
-					next_ip_delay = 90000.0/opt_frame_rate;
+					next_ip_delay = 90000.0/encparams.frame_rate;
 				else
 				{
 					if (!picture.topfirst)
-						next_ip_delay = 90000.0*2.0/opt_frame_rate;
+						next_ip_delay = 90000.0*2.0/encparams.frame_rate;
 					else
-						next_ip_delay = 90000.0*3.0/opt_frame_rate;
+						next_ip_delay = 90000.0*3.0/encparams.frame_rate;
 				}
 			}
 			else
 			{
-				if (opt_fieldpic)
-					next_ip_delay = 90000.0/(2.0*opt_frame_rate);
+				if (encparams.fieldpic)
+					next_ip_delay = 90000.0/(2.0*encparams.frame_rate);
 				else
 				{
 					if (!picture.repeatfirst)
-						next_ip_delay = 90000.0*2.0/(2.0*opt_frame_rate);
+						next_ip_delay = 90000.0*2.0/(2.0*encparams.frame_rate);
 					else
-						next_ip_delay = 90000.0*3.0/(2.0*opt_frame_rate);
+						next_ip_delay = 90000.0*3.0/(2.0*encparams.frame_rate);
 				}
 			}
 		}
@@ -1205,9 +1037,9 @@ void OnTheFlyRateCtl::CalcVbvDelay(Picture &picture)
 	{
 		/* first call of calc_vbv_delay */
 		/* we start with a 7/8 filled VBV buffer (12.5% back-off) */
-		picture_delay = ((opt_vbv_buffer_size*7)/8)*90000.0/opt_bit_rate;
-		if (opt_fieldpic)
-			next_ip_delay = (int)(90000.0/opt_frame_rate+0.5);
+		picture_delay = ((encparams.vbv_buffer_size*7)/8)*90000.0/encparams.bit_rate;
+		if (encparams.fieldpic)
+			next_ip_delay = (int)(90000.0/encparams.frame_rate+0.5);
 	}
 
 	/* VBV checks */
@@ -1224,11 +1056,11 @@ void OnTheFlyRateCtl::CalcVbvDelay(Picture &picture)
 
 	/* check for underflow (previous picture).
 	*/
-	if (!opt_low_delay && (decoding_time < (double)bitcnt_EOP*90000.0/opt_bit_rate))
+	if (!encparams.low_delay && (decoding_time < (double)bitcnt_EOP*90000.0/encparams.bit_rate))
 	{
 		/* picture not completely in buffer at intended decoding time */
 		mjpeg_warn("vbv_delay underflow frame %d (target=%.1f, actual=%.1f)",
-				   frame_num-1, decoding_time, bitcnt_EOP*90000.0/opt_bit_rate);
+				   frame_num-1, decoding_time, bitcnt_EOP*90000.0/encparams.bit_rate);
 	}
 
 
@@ -1246,7 +1078,7 @@ void OnTheFlyRateCtl::CalcVbvDelay(Picture &picture)
 	if ( decoding_time * ((double)bit_rate  / 90000.0) - ((double)bitcnt_EOP)
 		> vbv_buffer_size )
 	{
-		double oversize = opt_vbv_buffer_size -
+		double oversize = encparams.vbv_buffer_size -
 			(decoding_time / 90000.0 * bit_rate - (double)(bitcnt_EOP+frame_undershoot));
 		if(!quiet || oversize > 0.0  )
 			mjpeg_warn("vbv_delay overflow frame %d - %f.0 bytes!", 
@@ -1258,8 +1090,8 @@ void OnTheFlyRateCtl::CalcVbvDelay(Picture &picture)
 
 #ifdef OUTPUT_STAT
 	fprintf(statfile,
-			"\nvbv_delay=%d (bitcount=%lld, decoding_time=%.2f, bitcnt_EOP=%lld)\n",
-			picture.vbv_delay,bitcount(),decoding_time,bitcnt_EOP);
+			"\nvbv_delay=%d (coder.BitCount=%lld, decoding_time=%.2f, bitcnt_EOP=%lld)\n",
+			picture.vbv_delay,coder.BitCount(),decoding_time,bitcnt_EOP);
 #endif
 
 	if (picture.vbv_delay<0)
@@ -1277,10 +1109,20 @@ void OnTheFlyRateCtl::CalcVbvDelay(Picture &picture)
 		picture.vbv_delay = 65535;
 	}
 #else
-	if( !opt_mpeg1 || ctl_quant_floor != 0 || opt_still_size > 0)
+	if( !encparams.mpeg1 || encparams.quant_floor != 0 || encparams.still_size > 0)
 		picture.vbv_delay =  0xffff;
-	else if( opt_still_size > 0 )
-		picture.vbv_delay =  static_cast<int>(90000.0/opt_frame_rate/4);
+	else if( encparams.still_size > 0 )
+		picture.vbv_delay =  static_cast<int>(90000.0/encparams.frame_rate/4);
 #endif
 
 }
+
+
+
+/* 
+ * Local variables:
+ *  c-file-style: "stroustrup"
+ *  tab-width: 4
+ *  indent-tabs-mode: nil
+ * End:
+ */
