@@ -74,11 +74,11 @@ MultiplexJob::MultiplexJob()
     mux_format = MPEG_FORMAT_MPEG1;
     multifile_segment = false;
     always_system_headers = false;
-    packets_per_pack = 20;
+    packets_per_pack = 1;
     max_timeouts = 10;
     max_PTS = 0;
     emul_vcdmplex = 0;
-    max_segment_size = 2000; // MB, Safe default suitable for DVD etc...
+    max_segment_size = 0; // MB, default is unlimited (suitable for DVD)
     outfile_pattern = 0;
     packets_per_pack = 1;
 
@@ -139,7 +139,7 @@ void MultiplexJob::Usage(char *str)
 	"--system-headers|-h\n"
     "  Create System header in every pack in generic formats\n"
 	"--max-segment-size|-S size\n"
-    "  Maximum size of output file(s) in Mbyte (default: 2000) (0 = no limit)\n"
+    "  Maximum size of output file(s) in Mbyte (default: 0) (no limit)\n"
 	"--split-segment|-M\n"
     "  Simply split a sequence across files rather than building run-out/run-in\n"
 	"--help|-?\n"
