@@ -22,12 +22,20 @@
 
 #include "config.h"
 #include <limits.h>
+#include <stdlib.h>
 #include <math.h>
 #include "cpu_accel.h"
 #include "fastintfns.h"
 #include "motionsearch.h"
 #include "mjpeg_logging.h"
 
+/*
+ * Define prototypes so as to avoid warning errors at compile time
+*/
+
+void sub_mean_reduction(me_result_set *, int, int *);
+void subsample_image(uint8_t *,int ,uint8_t *,uint8_t *);
+void variance(uint8_t *,int ,int, unsigned int *,unsigned int *);
 
 /* The AltiVec code needs access to symbols during benchmarking
  * and verification.
@@ -399,8 +407,7 @@ STATIC void find_best_one_pel( me_result_set *sub22set,
 
 		matchrec = sub22set->mests[k];
 		orgblk = org + (i0+matchrec.x)+rowstride*(j0+matchrec.y);
-		penalty = intmax(intabs(matchrec.x),intabs(matchrec.y))<<5;
-
+		penalty = (intabs(matchrec.x)+intabs(matchrec.y))<<3;
 		for( i = 0; i < 4; ++i )
 		{
 			if( matchrec.x <= ilim && matchrec.y <= jlim )
